@@ -32,13 +32,13 @@ SortFilterModel::SortFilterModel(QObject *parent)
 {
     setObjectName(QStringLiteral("SortFilterModel"));
     setDynamicSortFilter(true);
-    connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            this, SIGNAL(countChanged()));
-    connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            this, SIGNAL(countChanged()));
-    connect(this, SIGNAL(modelReset()),
-            this, SIGNAL(countChanged()));
-    connect(this,  SIGNAL(countChanged()), this, SLOT(syncRoleNames()));
+    connect(this, &QAbstractItemModel::rowsInserted,
+            this, &SortFilterModel::countChanged);
+    connect(this, &QAbstractItemModel::rowsRemoved,
+            this, &SortFilterModel::countChanged);
+    connect(this, &QAbstractItemModel::modelReset,
+            this, &SortFilterModel::countChanged);
+    connect(this,  &SortFilterModel::countChanged, this, &SortFilterModel::syncRoleNames);
 }
 
 SortFilterModel::~SortFilterModel()
@@ -58,11 +58,18 @@ void SortFilterModel::syncRoleNames()
         m_roleIds[QString::fromUtf8(i.value())] = i.key();
     }
 
-    setRoleNames(sourceModel()->roleNames());
     setFilterRole(m_filterRole);
     setSortRole(m_sortRole);
 }
 
+QHash<int,QByteArray> SortFilterModel::roleNames() const
+{
+    if (sourceModel()) {
+       return sourceModel()->roleNames();
+    }
+    return {};
+}
+
 int SortFilterModel::roleNameToId(const QString &name) const
 {
     return m_roleIds.value(name, Qt::DisplayRole);
@@ -75,13 +82,13 @@ void SortFilterModel::setModel(QAbstractItemModel *model)
     }
 
     if (sourceModel()) {
-        disconnect(sourceModel(), SIGNAL(modelReset()), this, SLOT(syncRoleNames()));
+        disconnect(sourceModel(), &QAbstractItemModel::modelReset, this, &SortFilterModel::syncRoleNames);
     }
 
     QSortFilterProxyModel::setSourceModel(model);
 
     if (model) {
-        connect(model, SIGNAL(modelReset()), this, SLOT(syncRoleNames()));
+        connect(model, &QAbstractItemModel::modelReset, this, &SortFilterModel::syncRoleNames);
         syncRoleNames();
     }
 
@@ -172,7 +179,7 @@ void SortFilterModel::setSortRole(const QString &role)
         sort(-1, Qt::AscendingOrder);
     } else if (sourceModel()) {
         QSortFilterProxyModel::setSortRole(roleNameToId(role));
-        sort(0, sortOrder());
+        sort(sortColumn(), sortOrder());
     }
 }
 
@@ -183,7 +190,19 @@ QString SortFilterModel::sortRole() const
 
 void SortFilterModel::setSortOrder(const Qt::SortOrder order)
 {
-    sort(0, order);
+    if (order == sortOrder()) {
+        return;
+    }
+    sort(sortColumn(), order);
+}
+
+void SortFilterModel::setSortColumn(int column)
+{
+    if (column == sortColumn()) {
+        return;
+    }
+    sort(column, sortOrder());
+    emit sortColumnChanged();
 }
 
 QVariantMap SortFilterModel::get(int row) const
@@ -217,7 +236,7 @@ int SortFilterModel::mapRowFromSource(int row) const
 
 DataModel::DataModel(QObject *parent)
     : QAbstractItemModel(parent),
-      m_dataSource(0),
+      m_dataSource(nullptr),
       m_maxRoleId(Qt::UserRole + 1)
 {
     //There is one reserved role name: DataEngineSource
@@ -226,12 +245,12 @@ DataModel::DataModel(QObject *parent)
     ++m_maxRoleId;
 
     setObjectName(QStringLiteral("DataModel"));
-    connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)),
-            this, SIGNAL(countChanged()));
-    connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)),
-            this, SIGNAL(countChanged()));
-    connect(this, SIGNAL(modelReset()),
-            this, SIGNAL(countChanged()));
+    connect(this, &QAbstractItemModel::rowsInserted,
+            this, &DataModel::countChanged);
+    connect(this, &QAbstractItemModel::rowsRemoved,
+            this, &DataModel::countChanged);
+    connect(this, &QAbstractItemModel::modelReset,
+            this, &DataModel::countChanged);
 }
 
 DataModel::~DataModel()
@@ -249,7 +268,8 @@ void DataModel::dataUpdated(const QString &sourceName, const QVariantMap &data)
         QVariantList list;
 
         if (!m_dataSource->data()->isEmpty()) {
-            foreach (const QString &key, m_dataSource->data()->keys()) {
+            const auto lst = m_dataSource->data()->keys();
+            for (const QString &key : lst) {
                 if (!m_sourceFilter.isEmpty() && m_sourceFilterRE.isValid() && !m_sourceFilterRE.exactMatch(key)) {
                     continue;
                 }
@@ -293,12 +313,13 @@ void DataModel::setDataSource(QObject *object)
     }
 
     if (m_dataSource) {
-        disconnect(m_dataSource, 0, this, 0);
+        disconnect(m_dataSource, nullptr, this, nullptr);
     }
 
     m_dataSource = source;
 
-    foreach (const QString &key, m_dataSource->data()->keys()) {
+    const auto keys = m_dataSource->data()->keys();
+    for (const QString &key : keys) {
         dataUpdated(key, m_dataSource->data()->value(key).value<Plasma::DataEngine::Data>());
     }
 
@@ -389,7 +410,7 @@ void DataModel::setItems(const QString &sourceName, const QVariantList &list)
 
     if (!list.isEmpty()) {
         if (list.first().canConvert<QVariantMap>()) {
-            foreach (const QVariant &item, list) {
+            for (const QVariant &item : list) {
                 const QVariantMap &vh = item.value<QVariantMap>();
                 QMapIterator<QString, QVariant> it(vh);
                 while (it.hasNext()) {
@@ -403,7 +424,7 @@ void DataModel::setItems(const QString &sourceName, const QVariantList &list)
                 }
             }
         } else {
-            foreach (const QVariant &item, list) {
+            for (const QVariant &item : list) {
                 const QVariantMap &vh = item.value<QVariantMap>();
                 QMapIterator<QString, QVariant> it(vh);
                 while (it.hasNext()) {
@@ -419,8 +440,6 @@ void DataModel::setItems(const QString &sourceName, const QVariantList &list)
         }
     }
 
-    setRoleNames(m_roleNames);
-
     if (firstRun) {
         endResetModel();
     } else if (delta > 0) {
@@ -432,6 +451,11 @@ void DataModel::setItems(const QString &sourceName, const QVariantList &list)
                      createIndex(sourceIndex + qMin(list.length(), oldLength), 0));
 }
 
+QHash<int, QByteArray> DataModel::roleNames() const
+{
+    return m_roleNames;
+}
+
 void DataModel::removeSource(const QString &sourceName)
 {
     //FIXME: find a way to remove only the proper things also in the case where sources are items
